﻿
bl_info = {
	"name": "Image receiver",
	"author": "Warotaro Kashiwazaki",
	"version": (0, 1, 0),
	"blender": (2, 67, 0),
	"description": "Receives image over a WebSocket connection.",
	"category": "User",
	"support": "TESTING"
}

# importing and reloading modules
if "init_modules" in locals():
	import imp
	imp.reload(websocket_server)
else:
	from .websocket_server import WebSocketServer
		
init_modules = True

import bpy
from bpy.app.handlers import persistent
from bpy.props import BoolProperty, EnumProperty, IntProperty, PointerProperty, StringProperty
from bpy.types import AddonPreferences, Operator, Panel, PropertyGroup, USERPREF_HT_header, WindowManager

import bgl
import json
import numpy as np
import os

# Addon registration 
def register():

	bpy.utils.register_module(__name__)

	addon_prefs = bpy.context.user_preferences.addons[__name__].preferences
	
	if bool(addon_prefs.auto_start):

		start_server()

def unregister():

	_WebSocketServer.stop_server()

	bpy.utils.unregister_module(__name__)


# Addon core functions
def start_server():

	def started_event():

		print("  connected")
	
		send_message = json.JSONEncoder().encode(
			{ "type": "initialize" }
		)
		_WebSocketServer.broadcast(send_message)


	def received_message(message):

		if message.is_text:

			readMessageWebSocket(message)

		else:
		
			readImageFromWebSocket(message)

			updateImageViews()

	_WebSocketServer.set_listener(started_event, received_message)

	_WebSocketServer.start_server()

def stop_server():

	_WebSocketServer.stop_server()

def readMessageWebSocket(message):

	text = message.data.decode(message.encoding)
	print("  message: " + text)


def readImageFromWebSocket(message):

	print("  data: binary data (" + str(len(message)) + " bytes)")

	image_name = "image1"
	if image_name not in bpy.data.images:
		print("  new image created: " + image_name)
		img = bpy.data.images.new(image_name, 512, 512, alpha=True)
		img.generated_color = (0,0,0,0)
		img.use_fake_user = True

	print("  buffer to uint8 array: " + image_name)
	uint8_array = np.frombuffer(message.data, dtype='uint8')
	float_array = uint8_array.astype(np.float32)
	float_array[0::1] /= 255.0

	print("  store image data: " + image_name)
	blimg = bpy.data.images[image_name]
	blimg.pixels = float_array

	print("  bindcode: " + str(blimg.bindcode))
	if blimg.bindcode != 0:
		# must needs for update texture in 3DView
		print("  update gl texture: " + image_name)
		blimg.gl_free() 
		blimg.gl_load()

	print("  image size: " + str(len(float_array)) + "")

def updateImageViews():

	for window in bpy.context.window_manager.windows:

		for area in window.screen.areas: 

			if area.type in ['IMAGE_EDITOR', 'VIEW_3D']:

				print("view" + area.type)
				area.tag_redraw()

# WebSocketServer instance
_WebSocketServer = WebSocketServer()

# Addon preference UI
class WebGLRenderPreference(AddonPreferences):
	bl_idname = __name__

	auto_start = BoolProperty(
		name="Start automatically",
		description="Automatically start the server when loading the add-on",
		default=True
	)

	def draw(self, context):

		layout = self.layout
		row = layout.row()
		col = row.column()
		col.prop(self, "auto_start")

		row = layout.row()
		split = row.split(percentage=0.5)
		col = split.column()
		col.operator(Button_StartServer.bl_idname, icon='NEW', text="Start server")
		col = split.column()
		col.operator(Button_StopServer.bl_idname, icon='CANCEL', text="Stop server")

		row = layout.row()
		split = row.split(percentage=0.5)
		col = split.column()
		col.operator(Button_OpenClient.bl_idname, icon='LINK', text="Open client page")
		col = split.column()
		col.operator(Button_ToggleConsole.bl_idname, icon='LINK', text="Toggle console")

		row = layout.row()
		split = row.split(percentage=0.5)
		col = split.column()
		col.operator(Button_Test.bl_idname, icon='LINK', text="Send a test message to client")

class Button_StartServer(Operator):
	"""Start WebSocketServer"""
	bl_idname = "webgl_server.start"
	bl_label = "Start WebSocketServer"

	def execute(self, context):

		start_server()

		return {"FINISHED"}
	
class Button_StopServer(Operator):
	"""Stop WebSocketServer"""
	bl_idname = "webgl_server.stop"
	bl_label = "Stop WebSocketServer"

	def execute(self, context):

		stop_server()

		return {"FINISHED"}

class Button_OpenClient(Operator):
	"""test"""
	bl_idname = "webgl_server.open_client"
	bl_label = "Test"

	def execute(self, context):

		print("WebSocketServer open_client")

		script_file_path = os.path.realpath(__file__)
		directory_name = os.path.dirname(script_file_path)

		print("Dir: " + directory_name)

		#command = "explorer \"" + directory_name + "\\usage.html\""
		command = "explorer \"C:\\pg\\web\\experiments\\blender_webgl_render\\index.html\""

		print("Cmd: " + command)

		os.system(command)
				
		return {"FINISHED"}

class Button_ToggleConsole(Operator):
	"""ToggleConsole"""
	bl_idname = "webgl_server.toggle_console"
	bl_label = "Test"

	def execute(self, context):

		bpy.ops.wm.console_toggle()

		return {"FINISHED"}

class Button_Test(Operator):
	"""test"""
	bl_idname = "webgl_server.test"
	bl_label = "Test"

	def execute(self, context):

		#sending message test
		message = json.JSONEncoder().encode(
			{ "type": "text" }
		)
		_WebSocketServer.broadcast(message)
		return {"FINISHED"}

		#image writing test
		blimg = bpy.data.images['image1']
		width, height = blimg.size
		pixel_count = width * height
		pixels = [1.0]  * 4 * pixel_count
		blimg.pixels = pixels
		return {"FINISHED"}

if __name__ == "__main__":
	register()
